#include <iostream>
#include <algorithm>
#include <exception>
#include <string>
#include <algorithm>

//      - 128 
#define MAX_SIZE 128

//     
enum figure_type
{
    KING,
    QUEEN,
    ROOK,
    BISHOP,
    KNIGHT
};

//       
class chess_board
{
public:
    //  (  )
    chess_board(const int &size)
    {
        if (size >= 0 && size <= MAX_SIZE)
        {
            size_ = size;
            clean_board();
        }

        else
        {
            std::cout << "wrong size" << std::endl;
            throw std::string("wrong size");
        }
    }


    //    (  )
    void clean_board()
    {
        //     
        figures_on_board_ = 0;
        //    0
        for (int i = 0; i < size_; ++i)
        {
            for (int j = 0; j < size_; ++j)
            {
                board_[i][j] = 0;
            }
        }
    }

    // ,   
    int get_size()
    {
        return size_;
    }
    // ,     
    int get_figures_number()
    {
        return figures_on_board_;
    }

    //,        
    //        
    bool put_figure (const figure_type& figure, const int& x, const int& y, const int& step)
    {
        if (board_[x][y] > 0 || board_[x][y] < -step)
        {
           //       -    ,  
            return false;
        }

        if (check_figure(figure, x, y))
        {
            board_[x][y] = step;
            ++figures_on_board_;
            return true;
        }
        else
        {
            return false;
        }
    }

    //     
    //      
    void remove_figure(const int& x, const int& y)
    {
        board_[x][y] *= -1;
        --figures_on_board_;
    }

    //    
    void print() const
    {
        using namespace std;
        cout << "==========================================" << endl;
        for (int i = 0; i < size_; ++i)
        {
            for (int j = 0; j < size_; ++j)
            {
                //    ,   "*",   -  "-"
                cout << (board_[i][j] > 0?"*":"-") << " ";
            }
            cout << endl;
        }
        cout << "==========================================" << endl;
    }

private:

    //,         
    bool check_figure(const figure_type& figure, const int& x, const int& y)
    {
        //       
        switch (figure)
        {
            case KING:
                return check_king(x, y);
            case QUEEN:
                return check_queen(x, y);
            case ROOK:
                return check_rook(x, y);
            case BISHOP:
                return check_bishop(x, y);
            case KNIGHT:
                return check_knight(x, y);
                //     
            default:
                return false;
        }
    }

    //  
    bool check_king(const int& x, const int& y)
    {
        int min_x = std::max(x - 1, 0);
        int max_x = std::min(x + 1, size_ - 1);
        int min_y = std::max(y - 1, 0);
        int max_y = std::min(y + 1, size_ - 1);

        for (int i = min_x; i <= max_x; ++i)
        {
            for (int j = min_y; j <= max_y; ++j)
            {
                if (board_[i][j] > 0)
                {
                    return false;
                }
            }
        }
        return true;
    }

    //  
    bool check_rook(const int& x, const int& y)
    {
        for (int i = 0; i < size_; ++i)
        {
            if (board_[i][y] > 0)
            {
                return false;
            }
        }
        for (int j = 0; j < size_; ++j)
        {
            if (board_[x][j] > 0)
            {
                return false;
            }
        }
        return true;
    }

    //  
    bool check_bishop(const int& x, const int& y)
    {
        bool check = true;

        int k_start_1 = -std::min(x, y);
        int k_end_1 = std::min( size_ - x, size_ - y) - 1;

        int k_start_2 = -std::min( x, size_ - y - 1);
        int k_end_2 = std::min( y, size_ - x - 1);


        for (int k = k_start_1; k <= k_end_1; ++k)
        {
            check = check_coord(x + k, y + k) && check ;
        }
        for (int k = k_start_2; k <= k_end_2; ++k)
        {
            check = check_coord(x + k, y - k) && check;
        }

        return check;
    }


    //  
    bool check_knight(const int& x, const int& y)
    {
        bool check =
            check_coord(x - 2, y - 1) &&
            check_coord(x - 2, y + 1) &&
            check_coord(x - 1, y - 2) &&
            check_coord(x - 1, y + 2) &&
            check_coord(x + 1, y - 2) &&
            check_coord(x + 1, y + 2) &&
            check_coord(x + 2, y - 1) &&
            check_coord(x + 2, y + 1);
        return check;
    }

    //  
    bool check_queen(const int& x, const int& y)
    {
        return check_rook(x, y) && check_bishop(x, y);
    }

   //  ,  ,    - ,  
   //   
    bool check_coord(const int& x, const int& y)
    {
        if (x >= 0 && x < size_ && y >= 0 && y < size_)
        {
            if (board_[x][y] > 0)
            {
                return false;
            }
        }
        return true;
    }

    // if board_[x][y] == 0
    //      
    // if board_[x][y] == s
    //          S
    // if board_[x][y] == -s
    //      ,        S
    int board_[MAX_SIZE][MAX_SIZE];
    int size_;
    int figures_on_board_;
};

//          
int get_max_figures (const figure_type& figure, const int& size)
{
    switch (figure)
    {
        case KING:
            return (size + 1) / 2;
        case QUEEN:
            if (size <= 2)
            {
                return 1;
            }
            else if (size == 3)
            {
                return 2;
            }
            else
            {
                return size;
            }
        case ROOK:
            return size;
        case BISHOP:
            if (size == 1)
            {
                return size;
            }
            else
            {
                return 2 * (size - 1);
            }
        case KNIGHT:
            if (size <= 2)
            {
                return size * size;
            }
            else
            {
                return (size * size + 1) / 2;
            }
        default:
            return 1;
    }
}

//       
void place_figures_for_max(chess_board &board, const figure_type& figure, const int& max_figures, int step)
{
    ++step;
    const int size = board.get_size();
    for (int i = 0; i < size; ++i)
    {
        for (int j = 0; j < size; ++j)
        {
            //    ,         
            if (board.put_figure(figure, i, j, step))
            {
                //      ,   
                if (board.get_figures_number() == max_figures)
                {
                    board.print();
                }
                //     ,   
                else
                {
                    place_figures_for_max(board, figure, max_figures, step);
                }
                board.remove_figure(i, j);
            }
        }
    }
}

//,        
figure_type get_figure_type(const std::string& str)
{
    using namespace std;

    figure_type figure;
    if (str == string("king"))
    {
        figure = KING;}
    else if (str == string("queen"))
    {
        figure = QUEEN;}
    else if (str == string("rook"))
    {
        figure = ROOK;
    }
    else if (str == string("bishop"))
    {
        figure = BISHOP;
    }
    else if (str == string("knight"))
    {
       figure = KNIGHT;
    }
    //     
    else
    {
        cout << "wrong figure type" << endl;
        throw string("wrong figure type");
    }

    return figure;
}

int main()
{
    //   
    using namespace std;

    int size;
    string type_of_figure;
    string type_of_action;
    figure_type figure;

    cout << " : king, queen, rook, bishop, knight" << endl;
    cout << " : max, placing" << endl;
    cout << "----------------------------------------------------" << endl;

    cout <<"  : ";
    cin >> size;
    chess_board board(size);

    cout << "  : ";
    cin >> type_of_figure;
    figure = get_figure_type(type_of_figure);

    cout << "  : ";
    cin >> type_of_action;

    if (type_of_action == string("max"))
    {

        int max_figures = get_max_figures(figure, size);
        cout << "Max figures: " << max_figures << endl;

        cout << "     ? (yes/no) ";
        string answer;
        cin >> answer;
        //    ,       
        if (answer == string("yes"))
        {
            int step = 0;
            place_figures_for_max(board, figure, max_figures, step);
        }
    }
    //      ,     
    //  
    else if (type_of_action == string("placing"))
    {
        int figures_to_place;
        cout << "   : ";
        cin >> figures_to_place;

        int step = 0;
        place_figures_for_max(board, figure, figures_to_place, step);
    }
    //   
    else
    {
        cout << "wrong type of action" << endl;
        throw string("wrong type of action");
    }

    return 0;
}
